/* 
	-- Tick.js
	-- Tick
	
	-- Created by Kerry Usry on Mon, Jul 25, 2005
	-- Copyright (c) 2005 nimblepoint. All rights reserved.
*/


var xmlReq;
var requestType = 'https://';
var baseURL = 'tickspot.com'; // used for links
var getTaskURL = baseURL + '/api/get_open_tasks/'; // company. will be prepended
var addEntryURL = baseURL + '/api/add_entry/'; // company. will be prepended

var dashActive = true;
var timerRunning = false;
var timerPaused = false;
var timerEntry = false;
var timerEntryValid = false;
var timerSubmit = false;
var timeStringH;
var timeStringM;
var timeStringS;
var timeStringEnter;
var timeH;
var timeM;
var timeS;
var pauseH;
var pauseM;
var pauseS;

var buttonFadeTime = 1/3*1000;
var buttonMomentaryDisplayTime = 1/2*1000;

var kMinimumHeight = 93;
var kMinimumWidth =  339;
var kMinimumExpandedHeight = 93;
var kDefaultExpandedHeight = 400;

var rotatingWheelDiameter = 18;
var rotatingWheelDiameterDiv = rotatingWheelDiameter/2;
var currentRotatingWheelAngle = 0;
var rotatingWheelTimer;
var rotatingWheelCanvas;
var rotatingWheelContext;
var rotatingWheelImage;
var mathpi = Math.PI/10;

var widgetDisplayH;
var widgetDisplayC1;
var widgetDisplayM;
var widgetDisplayC2;
var widgetDisplayS;
var widgetDisplayMask;
var bottomDrawer;
var drawerProcess;
var drawerEntry;
var drawerError;
var playPauseObj;
var resetObj;
var urlInput;
var emailInput;
var passwordInput;


/*----------------------------------------------------------------------------------
	Initialize widget
----------------------------------------------------------------------------------*/
function initialize (event)
{
	widget.onshow = handleWidgetShown
	widget.onhide = handleWidgetHidden
	
	// assign elements
	widgetDisplayH = document.getElementById('timer_displayH');
	widgetDisplayC1 = document.getElementById('timer_displayC1');
	widgetDisplayM = document.getElementById('timer_displayM');
	widgetDisplayC2 = document.getElementById('timer_displayC2');
	widgetDisplayS = document.getElementById('timer_displayS');	
	widgetDisplayMask = document.getElementById('timer_displayMask');	
	bottomDrawer = document.getElementById('bottomDrawer');	
	drawerProcess = document.getElementById('drawerProcess');	
	drawerEntry = document.getElementById('drawerEntry');	
	drawerError = document.getElementById('drawerError');	
	drawerReport = document.getElementById('drawerReport');	
	playPauseObj = document.getElementById('playPauseButton');
	resetObj = document.getElementById('resetButton');
	urlInput = document.getElementById('url');
	emailInput = document.getElementById('email');
	passwordInput = document.getElementById('password');
	
	// create image buttons
	createButton('flipperButton', 'handleFlipperButtonClick');
	createButton('playPauseButton', 'handlePlayPauseClick');
	createButton('enterTimeButton', 'handleEnterTimeClick');
	createButton('submitTimeButton', 'handleSubmitTimeClick');
	createButton('submitCancelButton', 'handleSubmitCancelClick');
	createButton('reportCloseButton', 'handleSubmitCancelClick');
	createButton('reportTimecardButton', 'handlReportTimecardButton');
	createButton('resetButton', 'handleResetClick');
	
	// create form buttons
	gDoneButton = new AppleGlassButton (document.getElementById ('doneButton'), 'Done', handleDoneButtonClick);
	gDoneButton = new AppleGlassButton (document.getElementById ('cancelButton'), 'Cancel', handleCancelButtonClick);

	// set up rotating wheel
	timerText = new Fader(document.getElementById('timer_display'), null, 1/3*600);
	playPauseButton = new Fader(document.getElementById('playPauseButton'), null, 1/3*1000);
	wheelIcon = new Fader(document.getElementById('rotatingWheelCanvas'), null, 1/3*1000);
	flipperIcon = new Fader(document.getElementById('flipperIcon'), null, 1/3*1000);
	
	// set up rotating wheel
	rotatingWheelCanvas = document.getElementById('rotatingWheelCanvas');
	rotatingWheelContext = rotatingWheelCanvas.getContext('2d');
	rotatingWheelImage = new Image();
	rotatingWheelImage.src = 'Images/circle_running.png';
	rotatingWheelImage.onload = rotatingWheelImage.onerror = rotatingWheelImage.onabort = function() { updateRotatingWheel(false) }

	// get everything initially set
	if (playPauseButton) playPauseButton.fadeIn();
	saveTimerPrefs();
}

/*----------------------------------------------------------------------------------
	Functions for button actions
----------------------------------------------------------------------------------*/
function handlePlayPauseClick(event, buttonID)
{
	if (timerEntry)
	{
		timerEntry = false;
		shrink();
		resetDrawer();		
		
		startTimer ();
		timerPaused = false;
		
		playPauseObj.style.backgroundImage = 'url(Images/button_pause.png)';
		resetObj.style.backgroundImage = 'url(Images/button_reset_disabled.png)';
	}
	else if (timerPaused || !timerRunning)
	{
		startTimer ();
		timerPaused = false;
		
		playPauseObj.style.backgroundImage = 'url(Images/button_pause.png)';
		resetObj.style.backgroundImage = 'url(Images/button_reset_disabled.png)';
	}
	else
	{
		pauseH = timeH;
		pauseM = timeM;
		pauseS = timeS;

		stopTimer ();
		timerPaused = true;
		
		playPauseObj.style.backgroundImage = 'url(Images/button_play.png)';
		resetObj.style.backgroundImage = 'url(Images/button_reset.png)';
	}
}

function handleResetClick(event, buttonID)
{	
	if (timerPaused == true && (timerEntry == false || timerSubmit == true))
	{
		timerPaused = false;
		timerRunning = false;
		
		timeH = 0;
		timeM = 0;
		timeS = 0;
		pauseH = 0;
		pauseM = 0;
		pauseS = 0;
		
		resetObj.style.backgroundImage = 'url(Images/button_reset_disabled.png)';
		if (timerText) timerText.fadeOut();
	}
}

function handleFlipperButtonClick(event, buttonID)
{
	// set values from preferences
	urlInput.value = '';
	emailInput.value = '';
	passwordInput.value = '';
	if (Preference.get('url')) urlInput.value = Preference.get('url');
	if (Preference.get('email')) emailInput.value = Preference.get('email');
	if (Preference.get('password')) passwordInput.value = Preference.get('password');
	
	if (timerRunning && timerPaused == false && wheelIcon) wheelIcon.fadeOut();
	showWidgetBack();
}

function handleDoneButtonClick(event, buttonID)
{
	// set preferences
	Preference.set('url', trim(urlInput.value));
	Preference.set('email', trim(emailInput.value));
	Preference.set('password', trim(passwordInput.value));
	
	// reload front drawer if open
	if (timerEntry) {
		resetDrawer();
		openDrawer();
	}
	
	if (timerRunning && timerPaused == false && wheelIcon) wheelIcon.fadeIn();
	showWidgetFront();
	setButtonState(buttonID, 'normal');
}

function handleCancelButtonClick(event, buttonID)
{
	// reload front drawer if open
	if (timerEntry) {
		resetDrawer();
		openDrawer();
	}

	if (timerRunning && timerPaused == false && wheelIcon) wheelIcon.fadeIn();
	showWidgetFront();
	setButtonState(buttonID, 'normal');
}

function handleEnterTimeClick(event, buttonID)
{
	if (!timerEntry) 
	{
		bottomDrawer.style.background = "url('Images/drawer_stripe.png') no-repeat";
		timerEntry = true;
		expand();
		openDrawer();
	} 
	else
	{
		Preference.set('noteInput', document.getElementById('noteInput').value); // save notes
		timerEntry = false;
		shrink();
		resetDrawer();
		if (timerPaused) resetObj.style.backgroundImage = 'url(Images/button_reset.png)';
	}
}

function handleSubmitCancelClick(event, buttonID)
{
	timerEntry = false;
	shrink();
	resetDrawer();
	Preference.set('clientInput', '');
	Preference.set('projectInput', '');
	Preference.set('taskInput', '');
	Preference.set('noteInput', '');
	if (timerPaused) resetObj.style.backgroundImage = 'url(Images/button_reset.png)';
}

function handlReportTimecardButton()
{
	timecardURL = 'http://' + Preference.get('url') + '.' + baseURL;
	widget.openURL(timecardURL);
}

function handleSubmitTimeClick (event, buttonID)
{
	if (timerEntryValid)
	{
		var timestr = Preference.get('timeHours') + ':' + Preference.get('timeMins');
		Preference.set('noteInput', document.getElementById('noteInput').value);
	
		document.getElementById('submitTimeButton').style.visibility = 'hidden';
		document.getElementById('submitTimeButtonProcess').style.visibility = 'visible';
	
		xmlReq = new XMLHttpRequest();
		xmlReq.onreadystatechange = loadReportData;
		requestURL = requestType + Preference.get('url') + '.' + addEntryURL;
	
		xmlReq.open('POST', requestURL, true);
		xmlReq.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlReq.send('email='+Preference.get('email')+'&password='+Preference.get('password')+'&task_id='+Preference.get('taskInput')+'&hours='+timestr+'&notes='+escape(Preference.get('noteInput')));		
	}
}

/*----------------------------------------------------------------------------------
	Functions for timer
----------------------------------------------------------------------------------*/
function startTimer ()
{	
	// Initial start of timer.
	if (timerRunning == false) 
	{
		timerRunning = true;
		startTime = new Date().getTime();	
		
		// start with 00 seconds.		
		widgetDisplayH.innerText = '00';
		widgetDisplayC1.innerText = ':';
		widgetDisplayM.innerText = '00';
		widgetDisplayC2.innerText = ':';
		widgetDisplayS.innerText = '00';	
		widgetDisplayMask.style.Background = '#FFFFFF';	
		if (timerText) timerText.fadeIn();
		
		// set preferences
		Preference.set('timerRunning',timerRunning);
		Preference.set('startTime',startTime);
	}
	// Paused so recalc start time.
	if (timerPaused)
	{	
		startTime = new Date().getTime();
		var offsetH = pauseH * 3600000;
		var offsetM = pauseM * 60000;
		var offsetS = pauseS * 1000;
		startTime = startTime - offsetH - offsetM - offsetS;
	}
	timerID = window.setInterval(runtimeTimer,500);	
	startRotatingWheel();
}

function stopTimer ()
{	
	window.clearInterval(timerID);
	stopRotatingWheel();
}

function runtimeTimer ()
{	
	var elapsedTime;
	var Now = new Date().getTime();
	elapsedTime = Now - startTime;

	var afterH= elapsedTime % 3600000;
	timeH = (elapsedTime - afterH) / 3600000;
	var afterM= afterH % 60000;
	timeM = (afterH - afterM) / 60000;
	timeS = Math.floor(afterM / 1000);	

	updateTimerDisplay();
	
	// This is just a nice effect to hide the skip when the time updates after being hidden.
	if (dashActive == false) 
	{
		dashActive = true;
		if (timerText) timerText.fadeIn();
	}
}

function updateTimerDisplay ()
{
	var h = parseInt(timeH, 10);
	var m = parseInt(timeM, 10);
	var s = parseInt(timeS, 10);
	
	if (timerRunning == true) {
	
		// sec display
		if (s < 10) s = '0' + s;
		widgetDisplayS.innerText = s; 
		
		// min display
		if (m < 10) m = '0' + m;
		widgetDisplayM.innerText = m; 
		
		// hour display
		if (h < 10) h = '0' + h;
		widgetDisplayH.innerText = h; 
		
		// colon display
		widgetDisplayC1.innerText = ':'; 
		widgetDisplayC2.innerText = ':'; 
		
	} else {
	
		// reset display
		widgetDisplayH.innerText = '';
		widgetDisplayC1.innerText = '';
		widgetDisplayM.innerText = '';
		widgetDisplayC2.innerText = '';
		widgetDisplayS.innerText = '';	
		
	}
}

function saveTimerPrefs()
{
	// set preferences
	Preference.set('timerRunning',timerRunning);
	Preference.set('timerPaused',timerPaused);
	Preference.set('startTime',startTime);
	Preference.set('pauseH',pauseH);
	Preference.set('pauseM',pauseM);
	Preference.set('pauseS',pauseS);	
	Preference.set('timeH',timeH);
	Preference.set('timeM',timeM);
	Preference.set('timeS',timeS);
}

function getTimerPrefs()
{
	// set preferences
	timerRunning = Preference.get('timerRunning');
	timerPaused = Preference.get('timerPaused');
	startTime = Preference.get('startTime');
	pauseH = Preference.get('pauseH');
	pauseM = Preference.get('pauseM');
	pauseS = Preference.get('pauseS');	
	timeH = Preference.get('timeH');
	timeM = Preference.get('timeM');
	timeS = Preference.get('timeS');
}

function openDrawer()
{
	var requestURL;
	if (timerRunning) 
	{
		pauseH = timeH;
		pauseM = timeM;
		pauseS = timeS;
	
		stopTimer ();
		timerPaused = true;
		
		playPauseObj.style.backgroundImage = 'url(Images/button_play.png)';
		resetObj.style.backgroundImage = 'url(Images/button_reset_disabled.png)';			
	}

	xmlReq = new XMLHttpRequest();
	xmlReq.onreadystatechange = loadEntryData;
	requestURL = requestType + Preference.get('url') + '.' + getTaskURL;

	xmlReq.open('POST', requestURL, true);
	xmlReq.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlReq.send('email='+Preference.get('email')+'&password='+Preference.get('password'));	
}

function resetDrawer()
{
	drawerProcess.style.visibility = 'hidden';
	drawerError.style.visibility = 'hidden';
	drawerEntry.style.visibility = 'hidden';
	drawerReport.style.visibility = 'hidden';
	
	document.getElementById('submitTimeButton').style.visibility = 'hidden';
	document.getElementById('submitTimeButtonProcess').style.visibility = 'hidden';	
	document.getElementById('graphLeft').style.visibility = 'hidden';	
	document.getElementById('graphRight').style.visibility = 'hidden';	
}

function loadEntryData()
{
	if (xmlReq.readyState == 4)
	{
		if (xmlReq.status == 200)
		{
			drawerProcess.style.visibility = 'hidden';
			bottomDrawer.style.background = "url('Images/drawer_blue.png') no-repeat";
			drawerEntry.style.visibility = 'visible';
				
			// load initial client select data
			loadSelect('client', 'Client', 'client', 'client');
			if (Preference.get('clientInput') != '') loadSelect('project', 'Project', Preference.get('clientInput'), 'client');
			if (Preference.get('clientInput') != '' && Preference.get('projectInput') != '') loadSelect('task', 'Task', Preference.get('projectInput'), 'project');
		
			// set time value
			document.getElementById('timeInput').value = entryTimeString();
			document.getElementById('noteInput').value = '';
			if (Preference.get('noteInput') != undefined) document.getElementById('noteInput').value = Preference.get('noteInput');
			
			document.getElementById('submitTimeButton').style.visibility = 'visible';
		}
		else
		{
			drawerProcess.style.visibility = 'hidden';
			bottomDrawer.style.background = "url('Images/drawer_stripe.png') no-repeat";
			drawerError.style.visibility = 'visible';
		}
	}
}

function loadReportData()
{
	if (xmlReq.readyState == 4)
	{
		if (xmlReq.status == 200)
		{
			var total_hours = 0;
			var total_budget = 0;
			var total_users = 0;
			var budget_used = 0;
			var status_hours = 0;
			var graph_len = 238;
			var text_bar = 238;

			// reset the timer 
			timerSubmit = true;
			if (timerPaused) handleResetClick();
		
			var xml_results = xmlReq.responseXML;
			var xml_list = xml_results.getElementsByTagName('task');

			for (var i = 0; i < xml_list.length; i++)
			{
				var item_list = xml_list[i];
				if (item_list.getElementsByTagName('total_hours')[0].hasChildNodes()) {
					total_hours = item_list.getElementsByTagName('total_hours')[0].firstChild.nodeValue		
				}
				if (item_list.getElementsByTagName('budget')[0].hasChildNodes()) {
					total_budget = item_list.getElementsByTagName('budget')[0].firstChild.nodeValue
				}
				if (item_list.getElementsByTagName('user_count')[0].hasChildNodes()) {
					total_users = item_list.getElementsByTagName('user_count')[0].firstChild.nodeValue
				}
				if (total_budget != 0) budget_used = (total_hours / total_budget) * 100;
			}	
			
			// default values if all is in budget
			var status_color = 'green';
			var status_txt = 'hours remaining'
			
			if (total_budget != 0) {
				status_hours = Math.round((total_budget - total_hours)*100)/100;	
				total_budget = Math.round(total_budget*100)/100;
				graph_len = Math.round(budget_used * 2.38);
				text_bar = graph_len + 28;
			}
			total_hours = Math.round(total_hours*100)/100;
			
			// update css elements
			document.getElementById('graphText').style.color = '#2D9A20';
			document.getElementById('graphText').style.width = text_bar + 'px';			
			document.getElementById('graphMiddle').style.width = graph_len + 'px';
			document.getElementById('graphMiddle').style.borderRight = '1px solid #2D9A20';
			document.getElementById('graphLeft').style.visibility = 'visible';			
			document.getElementById('graphRight').style.visibility = 'hidden';
			
			// thing change a little when close and over budget, or no budget
			if (total_budget == 0)
			{
				document.getElementById('graphText').style.textAlign = 'center';
				document.getElementById('graphText').style.width = '238px';					
				document.getElementById('graphMiddle').style.width = '0px';
				document.getElementById('graphMiddle').style.borderRight = 'none';
				document.getElementById('graphLeft').style.visibility = 'hidden';			
			}
			else if (budget_used > 100) 
			{
				status_color = 'red';
				status_txt = 'hours over'
				status_hours = Math.round((total_hours - total_budget)*100)/100;
				
				document.getElementById('graphText').style.textAlign = 'center';
				document.getElementById('graphText').style.color = '#FFFFFF';
				document.getElementById('graphText').style.width = '238px';					
				document.getElementById('graphMiddle').style.width = '238px';
				document.getElementById('graphMiddle').style.borderRight = '1px solid #971518';
				document.getElementById('graphRight').style.visibility = 'visible';
				
			} 
			else if (budget_used > 75) 
			{
				var status_color = 'yellow';
				var text_bar = graph_len - 6;
				
				document.getElementById('graphText').style.color = '#FFFFFF';
				document.getElementById('graphText').style.width = text_bar + 'px';					
				document.getElementById('graphMiddle').style.borderRight = '1px solid #CC8500';
			}
			
			// set time entered
			var displayMins = Preference.get('timeMins');
			if (displayMins == 0) displayMins = '00';
			document.getElementById('reportHours').innerHTML = Preference.get('timeHours');
			document.getElementById('reportMins').innerHTML = Preference.get('timeMins');
			
			// load up report data before showing
			document.getElementById('reportClient').innerHTML = Preference.get('clientInputTxt');
			document.getElementById('reportProject').innerHTML = Preference.get('projectInputTxt');
			document.getElementById('reportTask').innerHTML = Preference.get('taskInputTxt');
			
			document.getElementById('reportBudgetPeople').innerHTML = total_users;
			if (total_budget == 0) {
				document.getElementById('graphText').innerHTML = 'This task has no budget';
				document.getElementById('reportBudgetHours').innerHTML = 'Total time entered: <b>'+total_hours+' hours</b>';
			} else {
				document.getElementById('graphText').innerHTML = Math.round(budget_used) + '%';
				document.getElementById('reportBudgetHours').innerHTML = 'Task Budget: <b>'+status_hours+' '+status_txt+'</b> (of '+total_budget+')';
			}
			
			// change all the elements with status color
			bottomDrawer.style.backgroundImage = 'url(Images/drawer_'+status_color+'.png)';
			document.getElementById('graphBg').style.backgroundImage = 'url(Images/graph_bg_'+status_color+'.png)';
			document.getElementById('graphLeft').style.backgroundImage = 'url(Images/graph_left_'+status_color+'.png)';
			document.getElementById('graphMiddle').style.backgroundImage = 'url(Images/graph_middle_'+status_color+'.png)';
			document.getElementById('graphRight').style.backgroundImage = 'url(Images/graph_right_'+status_color+'.png)';
			
			reportButtonBg.style.backgroundImage = 'url(Images/button_bg_'+status_color+'.png)';
			reportTimecardButton.style.backgroundImage = 'url(Images/button_timecard_'+status_color+'.png)';
			reportCloseButton.style.backgroundImage = 'url(Images/button_close_'+status_color+'.png)';
			
			// now that all is loaded switch out the views
			drawerProcess.style.visibility = 'hidden';
			drawerEntry.style.visibility = 'hidden';
			drawerReport.style.visibility = 'visible';
			
			document.getElementById('timeInput').value = '';
			document.getElementById('submitTimeButtonProcess').style.visibility = 'hidden';	
			
			// now clean up and reset everything
			timerSubmit = false;
			timerEntryValid = false;
			Preference.set('clientInput', '');
			Preference.set('projectInput', '');
			Preference.set('taskInput', '');
			Preference.set('noteInput', '');
			Preference.set('timeHours', '');	
			Preference.set('timeMins', '');		
		}
		else
		{
			drawerEntry.style.visibility = 'hidden';
			bottomDrawer.style.background = "url('Images/drawer_stripe.png') no-repeat";
			drawerError.style.visibility = 'visible';
		}
	}
}

function loadSelect(elementKey, elementDisplay, parentID, parentKey) 
{
	if (parentKey == 'client')
	{
		document.getElementById('projectInputText').innerHTML = '';
		document.getElementById('projectInput').length = 0;
		document.getElementById('taskInputText').innerHTML = '';
		document.getElementById('taskInput').length = 0;
		document.getElementById('taskInput').value = '';
	}
	
	if (parentKey == 'project')
	{
		document.getElementById('taskInputText').innerHTML = '';
		document.getElementById('taskInput').length = 0;
		document.getElementById('taskInput').value = '';
	}

	var selectObj = document.getElementById(elementKey + 'Input');
	selectObj.length = 0;
	if (parentID != '' && elementKey != 'task') selectObj.options[selectObj.length] = new Option('// Existing ' + elementDisplay + 's //', '');
	
	var xml_results = xmlReq.responseXML;
	var xml_list = xml_results.getElementsByTagName(elementKey);
	
	for (var i = 0; i < xml_list.length; i++)
	{
		var item_list = xml_list[i];
		if ((parentID == 'client') || (parentID == item_list.getElementsByTagName(parentKey + '_id')[0].firstChild.nodeValue))
		{
			item_id = item_list.getElementsByTagName(elementKey + '_id')[0].firstChild.nodeValue
			item_name = item_list.getElementsByTagName(elementKey + '_name')[0].firstChild.nodeValue
			selectObj.options[selectObj.length] = new Option(item_name, item_id);
		}
	}	
	// check for valid entry
	checkEntry();	
	
	// set and reload from preferences
	if (selectObj.options.length != 0) selectObj.value = Preference.get(elementKey + 'Input');
	selectChange(elementKey+'Input');
}

function selectChange(selectName)
{
	var selectObj = document.getElementById(selectName);
	document.getElementById(selectName+'Text').innerHTML = selectObj.options[selectObj.selectedIndex].text;
	Preference.set(selectName+'Txt', selectObj.options[selectObj.selectedIndex].text);	
	Preference.set(selectName,selectObj.value);	
}

function checkEntry()
{
	var timestr = document.getElementById('timeInput').value;
	var taskid = document.getElementById('taskInput').value;

	if (parseTimeStr(timestr) && taskid != '')
	{
		timerEntryValid = true;
		submitTimeButton.style.backgroundImage = 'url(Images/button_submit_time.png)';
	}
	else
	{
		timerEntryValid = false;
		submitTimeButton.style.backgroundImage = 'url(Images/button_submit_time_disabled.png)';
	}
}

function entryTimeString()
{
	var timestring;
	if (pauseH > 0 || pauseM > 0 || pauseS > 0)
	{
		var h = parseInt(pauseH, 10);
		var m = parseInt(pauseM, 10);
		var s = parseInt(pauseS, 10);
		
		if (s > 30) {
			m = m + 1; s = 0;
		}
		if (m > 60) {
			h = h + 1; m = 0;
		}
		
		if (m < 10) m = '0' + m;
		timestring = h + ':' + m;
	} 
	else
	{
		timestring = '0:00';
	}
	return timestring;
}

function parseTimeStr(time) 
{
	var hours = 0;
	var mins = 0;
	var timesep = '';
	var addzero = '';
	time = trim(time);
	
	if (time.indexOf(":") != -1) timesep = ":";
	if (time.indexOf(".") != -1) timesep = ".";
	if (time.indexOf(",") != -1) timesep = ",";
	
	if (timesep != "")
	{
		if (time.indexOf(timesep) + 1 == time.length - 1) addzero = '0';
		var timearr = time.split(timesep);
		hours = timearr[0];
		mins = timearr[1];
		
		// check limits on entry
		if (mins.length > 2) mins = mins.substr(0, 2);
		if (timesep == ":" && mins.length < 2) mins = mins + '0';
		if (timesep == ":" && mins > 59) mins = '00';
		
		// do not allow more than 24 hours
		if ((hours > 24) || (hours == 24 && mins > 0)) 
		{
			hours = 24;
			mins = '00';
		}		
		
		document.getElementById('timeInput').value = hours + timesep + mins;
		mins = mins + addzero;
		if (timesep == "." || timesep == ",") mins = mins * .6;	
	}
	else if (!isNaN(time))
	{
		hours = time;
		if (hours > 24) hours = 24;
		document.getElementById('timeInput').value = hours;
	}

	if (hours < 0 || hours == '') hours = 0;
	if (mins < 0 || mins == '') mins = 0;
	
	if (hours == 0 && mins == 0)
	{
		return false;	
	}
	else
	{
		if (mins < 10 && mins.length == 1) mins = '0' + mins;
		Preference.set('timeHours', hours);	
		Preference.set('timeMins', mins);	
		return true;
	}
}

/*----------------------------------------------------------------------------------
	Functions for animating rotating wheel
----------------------------------------------------------------------------------*/
function startRotatingWheel()
{
	if (!rotatingWheelTimer)
	{
		var wheelObj = document.getElementById('rotatingWheelCanvas');
		wheelObj.style.visibility = 'visible';
		if (wheelIcon) wheelIcon.fadeIn();
		rotatingWheelTimer = setInterval('updateRotatingWheel()', 1/15*700);
	}
}

function stopRotatingWheel()
{
	if (rotatingWheelTimer)
	{
		if (wheelIcon) wheelIcon.fadeOut();
		clearInterval(rotatingWheelTimer);
		rotatingWheelTimer = null;
	}
}

function updateRotatingWheel()
{
	currentRotatingWheelAngle += mathpi;
	rotatingWheelContext.clearRect(0, 0, rotatingWheelDiameter, rotatingWheelDiameter);
	rotatingWheelContext.save();
	rotatingWheelContext.translate(rotatingWheelDiameterDiv, rotatingWheelDiameterDiv);
	rotatingWheelContext.rotate(currentRotatingWheelAngle);
	rotatingWheelContext.translate(-rotatingWheelDiameterDiv, -rotatingWheelDiameterDiv);
	rotatingWheelContext.drawImage(rotatingWheelImage, 0, 0, rotatingWheelDiameter, rotatingWheelDiameter);
	rotatingWheelContext.restore();
}

/*----------------------------------------------------------------------------------
	Functions for handling widget flipping
----------------------------------------------------------------------------------*/
function showWidgetFront ()
{
	widget.prepareForTransition('ToFront')
	
	var widgetFront = document.getElementById('widgetFront');
	var widgetBack = document.getElementById('widgetBack');
	
	widgetBack.style.display = 'none';
	widgetFront.style.display = 'block';
	
	setTimeout('widget.performTransition()', 0);
	updateTime ();
}

function showWidgetBack()
{
	var currentSize = Window.currentSize();
	if (currentSize.y < kDefaultExpandedHeight) window.resizeTo(currentSize.x,kDefaultExpandedHeight);

	widget.prepareForTransition('ToBack');
	setButtonState('flipperButton', 'normal');
	
	var widgetFront = document.getElementById('widgetFront');
	var widgetBack = document.getElementById('widgetBack');
	
	widgetFront.style.display = 'none';
	widgetBack.style.display = 'block';
	
	setTimeout('widget.performTransition()', 0);
	if (flipperIcon) flipperIcon.fadeOut();
}

function createButton(buttonID, actionFunction)
{
	setButtonState(buttonID, 'normal');

	var button = document.getElementById(buttonID);

	button.setAttribute('onmouseover', 'handleButtonMouseOver(event, "'+buttonID+'")');
	button.setAttribute('onmouseout', 'handleButtonMouseOut(event, "'+buttonID+'")');
	button.setAttribute('onmousedown', 'handleButtonMouseDown(event, "'+buttonID+'")');
	button.setAttribute('onclick', 'handleButtonClick(event, "'+buttonID+'", '+actionFunction+')');
}

function buttonState(buttonID)
{
	var button = document.getElementById(buttonID);

	var classes = button.className.split(' ');

	for (var i = 0; i<classes.length; i++)
	{
		var currentClassName = classes[i];
		if (currentClassName.indexOf('buttonState_')==0);
			return classes[i].substr('buttonState_'.length);
	}

	return '';
}

function setButtonState(buttonID, state)
{
	var button = document.getElementById(buttonID);

	var classes = button.className.split(' ');

	var didButtonStateExist = false;

	for (var i = 0; i<classes.length; i++)
	{
		var currentClassName = classes[i];
		if (currentClassName.indexOf('buttonState_')==0)
		{
			classes[i] = 'buttonState_'+state;
			didButtonStateExist = true;
		}
	}

	if (!didButtonStateExist)
		classes.push('buttonState_'+state);

	button.className = classes.join(' ');
}

function handleButtonMouseOver(event, buttonID)
{
	if (buttonState(buttonID)!='disabled')
		setButtonState(buttonID, 'mouseOver');
}

function handleButtonMouseOut(event, buttonID)
{
	if (buttonState(buttonID)!='disabled')
		setButtonState(buttonID, 'normal');
}

function handleButtonMouseDown(event, buttonID)
{
	if (buttonState(buttonID)=='disabled')
		return;

	setButtonState(buttonID, 'pressed');

	event.stopPropagation();
	event.preventDefault();
}

function handleButtonClick(event, buttonID, actionFunction)
{
	if (buttonState(buttonID)=='disabled')
		return;

	setButtonState(buttonID, 'mouseOver');

	event.stopPropagation();
	event.preventDefault();

	actionFunction(event, buttonID);
}

function handleMouseMove(event)
{
	if (flipperIcon) flipperIcon.fadeIn();
}

function handleMouseOut(event)
{
	if (flipperIcon) flipperIcon.fadeOut();
}

/*----------------------------------------------------------------------------------
	Functions for handling widget shown and hidden states
----------------------------------------------------------------------------------*/
function handleWidgetShown(event)
{
	// Get the current state.
	getTimerPrefs();
	
	// If coming back after shutdown/restart and paused, sets timer text.
	if (timerPaused)
	{
		h = parseInt(pauseH, 10);
		m = parseInt(pauseM, 10);
		s = parseInt(pauseS, 10);
	
		// sec display
		if (s < 10) s = '0' + s;
		widgetDisplayS.innerText = s; 
		
		// min display
		if (m < 10) m = '0' + m;
		widgetDisplayM.innerText = m; 
		
		// hour display
		if (h < 10) h = '0' + h;
		widgetDisplayH.innerText = h; 	
		
		// add colons
		widgetDisplayC1.innerText = ':';
		widgetDisplayC2.innerText = ':';		
	}

	// Now get it going.
	if (timerRunning && timerPaused == false) 
	{
		startTimer();
	}
	else
	{
		// This is just a adds consistant fade in after coming back.
		dashActive = true;
		if (timerText && timerRunning) timerText.fadeIn();
	}	
}

function handleWidgetHidden(event)
{
	dashActive = false;
	if (timerRunning) stopTimer();
	
	// Save the current state.
	saveTimerPrefs();
	
	// This hides the skip when recalcing after coming back.
	if (timerText) timerText.fadeOut();
}

/*----------------------------------------------------------------------------------
	Preference & Localization Utilities
----------------------------------------------------------------------------------*/
var Preference = new Object();
Preference.instancePrefs = new Array();

Preference.get = function (key) {
	if (window.widget) {
		var str = widget.preferenceForKey(_createkey(key));
		return str;
	}
}

Preference.set = function (key,value) {
	if (window.widget) {
		var instanceKey = _createkey(key);
		widget.setPreferenceForKey(value, instanceKey);
		widget.setPreferenceForKey(value, key);
		Preference.instancePrefs[instanceKey] = true;
	}
}

Preference.removeInstancePreferences = function () {
	if (window.widget) {
		for (key in Preference.instancePrefs) {
			widget.setPreferenceForKey(null, key);
		}
	}
}

function _createkey(key) {
        return widget.identifier + '-' + key;
}

/*----------------------------------------------------------------------------------
	Formatting and Parsing functions
----------------------------------------------------------------------------------*/
function trim(str)
{
	str = str.replace(/^\s+/g, '');
	return str.replace(/\s+$/g, '');
}

/*----------------------------------------------------------------------------------
	Window resizing
----------------------------------------------------------------------------------*/
var Window = function () {
	// Window is created with the shrunk state (defined in Info.plist)
	Window.expanded = false;
	Window.shrunkHeight = window.innerHeight;		
		
	Window.page = document.getElementById('pageScroll');
	Window.pageStyle = Window.page.style;		
	Window.bottom = document.getElementById('bottomDrawer');
	Window.bottomStyle = Window.bottom.style;	
	Window.initialSize = new Position(kMinimumWidth,kMinimumHeight);
	
	var y = Preference.get('windowHeight');
	Window.expandedHeight = y ? y: kDefaultExpandedHeight;

	Window.resizeTo(new Position(kMinimumWidth,Window.initialSize.y),'initialize');
}

window.addEventListener('load', Window, false);

Window.currentSize = function () {
	return new Position(kMinimumWidth, window.innerHeight);
}

Window.resizeTo = function (newSize, status) {
	// limit the size
	if (newSize.x < kMinimumWidth) newSize.x = kMinimumWidth;
	if (!Window.expanded || (status == 'expanding')) {
		if (newSize.y < kMinimumHeight) newSize.y = kMinimumHeight;
	} else {
		if (newSize.y < kMinimumExpandedHeight) newSize.y = kMinimumExpandedHeight;
	}
	var currentSize = Window.currentSize();
	if (!newSize.equal(currentSize) || status != 'tracking') {

		if (Window.expanded) {
			window.resizeTo(newSize.x,newSize.y);
		}
		Window.pageStyle.height = (newSize.y) + 'px';
		Window.bottomStyle.height = (newSize.y - 80) + 'px';

		if (!Window.expanded) {
			window.resizeTo(newSize.x,currentSize.y);
		}

		// set the window size pref at the end
		if (status != 'trackingEnd' && Window.expanded) {
			Window.initialSize.set(newSize);
			Preference.set('windowHeight', newSize.y);
			Window.expandedHeight = newSize.y;
		}
	}
}


/*----------------------------------------------------------------------------------
	ExpandShrink Animation
----------------------------------------------------------------------------------*/
function expand (func) { ExpandShrink.animate(1, func); }
function shrink (func) { ExpandShrink.animate(0, func); }

var ExpandShrink = new Object ();

ExpandShrink.animator = function () {
	if ( ! ExpandShrink._animator) {
		var duration = 200;
		ExpandShrink._animator = new AppleAnimator (duration, 13);
		ExpandShrink._animator.oncomplete = ExpandShrink.onFinish;
	}
	return ExpandShrink._animator;
}

ExpandShrink.adjust = function (animator, now, first, done) {
	Window.resizeTo(new Position(kMinimumWidth, Math.round(now)), 'expanding');
	document.getElementById('bottomDrawer').style.visibility = 'visible';
}

ExpandShrink.onFinish = function () {
	if (ExpandShrink._animation) {
		delete ExpandShrink._animation;
		ExpandShrink._animation = null;
	}
	if (ExpandShrink._animator) {
		delete ExpandShrink._animator;
		ExpandShrink._animator = null;
	}

	if (Window.expanded) {
		// this has to happen here so that it does not appear before animation is done.
		document.getElementById('bottomDrawer').style.visibility = 'visible';
		drawerProcess.style.visibility = 'visible';
	} else {
		document.getElementById('bottomDrawer').style.visibility = 'hidden';
 	}
	if (ExpandShrink.callback) setTimeout(ExpandShrink.callback, 0);
}

ExpandShrink.animate = function (expand, callback) {
	if (Window.expanded == expand) return;
	Window.expanded = expand;

	ExpandShrink.callback = callback;
	var animator = ExpandShrink.animator();
	animator.oncomplete = ExpandShrink.onFinish;
	
	var from = expand ? kMinimumHeight: Window.currentSize().y;
	var to = expand ? Math.max(Window.expandedHeight, kMinimumExpandedHeight) : kMinimumHeight;
	
	ExpandShrink._animation = new AppleAnimation(from, to, ExpandShrink.adjust);
	animator.addAnimation(ExpandShrink._animation);
	animator.start();
}

/*----------------------------------------------------------------------------------
	Position object
----------------------------------------------------------------------------------*/
var Position = function (x,y) { this.x = x; this.y = y;}
Position.prototype = {
set : function (pos) { this.x = pos.x; this.y = pos.y;},
add : function (pos) { return new Position(this.x+pos.x, this.y+pos.y);},
sub : function (pos) { return new Position(this.x-pos.x, this.y-pos.y);},
equal : function (pos) { return (this.x==pos.x) && (this.y==pos.y); },
toString : function () { return '['+this.x+','+this.y+']';}
}

/*----------------------------------------------------------------------------------
	Debug
----------------------------------------------------------------------------------*/
function debug(msg) 
{
     if (!debug.box) {
          debug.box = document.createElement('div');
          debug.box.setAttribute('style', 
          	'background-color: white; ' +
          	'font-family: monospace; ' +
          	'font-size: 10px; ' +
          	'border: solid black 3px; ' +
          	'padding: 10px;');
          document.body.appendChild(debug.box);
          debug.box.innerHTML = '<h1 style="text-align:center">Debug Output</h1>';
     }
     
     var p = document.createElement('p');
     p.appendChild(document.createTextNode(msg));
     debug.box.appendChild(p);
}